﻿#region Copyright (C)
// ---------------------------------------------------------------------------------------------------------------
//  <copyright file="Form1.cs" company="Smurf-IV">
// 
//  Copyright (C) 2011-2012 Smurf-IV
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 2 of the License, or
//   any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program. If not, see http://www.gnu.org/licenses/.
//  </copyright>
//  <summary>
//  Url: http://Elucidate.codeplex.com/
//  Email: http://www.codeplex.com/site/users/view/smurfiv
//  </summary>
// --------------------------------------------------------------------------------------------------------------------
#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Media;
using System.Reflection;
using System.Windows.Forms;
using Microsoft.Win32.TaskScheduler;
using NLog;
using Shared;

namespace Elucidate
{
   public sealed partial class Elucidate : Form
   {
      private static readonly Logger Log = LogManager.GetCurrentClassLogger();
      private static readonly string userAppData = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), @"Elucidate");
      const string taskName = "SnapRAID Sync";

      public Elucidate()
      {
         InitializeComponent();
         if (Properties.Settings.Default.UpdateRequired)
         {
            // Thanks go to http://cs.rthand.com/blogs/blog_with_righthand/archive/2005/12/09/246.aspx
            Properties.Settings.Default.Upgrade();
            Properties.Settings.Default.UpdateRequired = false;
            Properties.Settings.Default.Save();
         }
         WindowLocation.GeometryFromString(Properties.Settings.Default.WindowLocation, this);
         // Add threading callbacks
         actionWorker.DoWork += actionWorker_DoWork;
         actionWorker.ProgressChanged += actionWorker_ProgressChanged;
         actionWorker.RunWorkerCompleted += actionWorker_RunWorkerCompleted;
         comboBox1.Text = "Stopped";
         comboBox1.Enabled = false;
         timer1.Enabled = true;
      }

      private void Form1_Shown(object sender, EventArgs e)
      {
         VersionIndicator.Text = Assembly.GetExecutingAssembly().GetName().Version.ToString();
         if (!Properties.Settings.Default.ConfigFileIsValid)
         {
            settingsToolStripMenuItem_Click(sender, e);
         }
         else
         {
            runWithoutCaptureMenuItem.Checked = Properties.Settings.Default.RunWithoutCapture;
            toolStripStatusLabel1.Text = DateTime.Now.ToString("u");
            EnableIfValid(Properties.Settings.Default.ConfigFileIsValid);
         }
      }

      private void Form1_FormClosing(object sender, FormClosingEventArgs e)
      {
         // persist our geometry string.
         Properties.Settings.Default.WindowLocation = WindowLocation.GeometryToString(this);
         Properties.Settings.Default.Save();
      }

      private void EnableIfValid(bool enabled)
      {
         btnDiff.Enabled = enabled;
         btnSync.Enabled = enabled;
         btnCheck.Enabled = enabled;
         EnableScheduleButtons(enabled);
      }

      #region Menu Handlers
      private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
      {
         if (actionWorker.IsBusy)
         {
            SystemSounds.Beep.Play();
            return;
         }
         toolStripStatusLabel1.Text = DateTime.Now.ToString("u");
         new Settings().ShowDialog(this);
         EnableIfValid(Properties.Settings.Default.ConfigFileIsValid);
         runWithoutCaptureMenuItem.Checked = Properties.Settings.Default.RunWithoutCapture;
      }

      private void logViewToolStripMenuItem_Click(object sender, EventArgs e)
      {
         if (actionWorker.IsBusy)
         {
            SystemSounds.Beep.Play();
            return;
         }
         toolStripStatusLabel1.Text = DateTime.Now.ToString("u");
         OpenFileDialog openFileDialog = new OpenFileDialog
         {
            InitialDirectory = Path.Combine(userAppData, @"Logs"),
            Filter = "Log files (*.log)|*.log|Archive logs (*.*)|*.*",
            FileName = "*.log",
            FilterIndex = 2,
            Title = "Select name to view contents"
         };

         if (openFileDialog.ShowDialog() == DialogResult.OK)
         {
            if (Properties.Settings.Default.UseWindowsSettings)
            {
               Process word = Process.Start("Wordpad.exe", '"' + openFileDialog.FileName + '"');
               if (word != null)
               {
                  word.WaitForInputIdle();
                  SendKeys.SendWait("^{END}");
               }
            }
            else
            {
               // Launch whatever "Knows" how to view log files
               Process.Start('"' + openFileDialog.FileName + '"');
            }
         }
      }


      private void changeLogLocationToolStripMenuItem_Click(object sender, EventArgs e)
      {
         if (actionWorker.IsBusy)
         {
            SystemSounds.Beep.Play();
            return;
         }
         new LogFileLocation().ShowDialog(this);
      }

      private void helpToolStripMenuItem_Click(object sender, EventArgs e)
      {
         Process.Start(@"http://elucidate.codeplex.com/documentation");
      }

      #endregion

      #region Command Button click actions

      private void btnDiff_Click(object sender, EventArgs e)
      {
         StartSnapRaidProcess("Diff");
      }

      private void btnSync_Click(object sender, EventArgs e)
      {
         StartSnapRaidProcess("Sync");
      }

      private void btnCheck_Click(object sender, EventArgs e)
      {
         StartSnapRaidProcess("Check");
      }

      #endregion

      #region Scheduling
      //
      // See the code in the following location for how to do these things
      // http://taskscheduler.codeplex.com/wikipage?title=Examples&referringTitle=Documentation
      //

      private void btnGetSchedules_Click(object sender, EventArgs e)
      {
         try
         {
            EnableScheduleButtons(false);
            toolStripStatusLabel1.Text = DateTime.Now.ToString("u");
            toolStripProgressBar1.DisplayText = "Starting";
            toolStripProgressBar1.State = ProgressState.Normal;
            toolStripProgressBar1.Style = ProgressBarStyle.Marquee;
            toolStripProgressBar1.Value = 0;
            UseWaitCursor = true;


            using (TaskService ts = new TaskService())
            {
               // Display version and server state
               Version ver = ts.HighestSupportedVersion;
               Log.Info("Highest version: {0}", ver);
               Log.Info("Server: {0} ({1})", ts.TargetServer, ts.Connected ? "Connected" : "Disconnected");
               // Output all the tasks in the root folder with their triggers and actions
               TaskFolder tf = ts.RootFolder;
               Log.Info("Root folder tasks ({0}):", tf.Tasks.Count);

               lstHistory.Tasks = tf.Tasks;
               //lstHistory.Tasks = ts.FindAllTasks();

            }
         }
         finally
         {
            toolStripProgressBar1.Style = ProgressBarStyle.Continuous;
            UseWaitCursor = false;
            toolStripProgressBar1.DisplayText = string.Empty;
            toolStripProgressBar1.Value = 0;
            EnableScheduleButtons(true);
         }
      }

      private void EnableScheduleButtons(bool b)
      {
         btnGetSchedules.Enabled = b;
         btnNew.Enabled = b;
         btnEdit.Enabled = b;
         btnDelete.Enabled = b;
      }

      private void GetTaskTemplate()
      {
         string user = System.Security.Principal.WindowsIdentity.GetCurrent().Name;
         //bool preWin7 = Environment.OSVersion.Version < new Version(6, 1);
         // Get the service on the local machine
         using (TaskService ts = new TaskService())
         {
            // Create a new task
            // Create a new task definition and assign properties
            TaskDefinition td = ts.NewTask();
            td.Data = "http://elucidate.codeplex.com/documentation";
            //td.Principal.UserId = user;
            //td.Principal.LogonType = TaskLogonType.InteractiveToken;
            td.RegistrationInfo.Author = "Elucidate";
            td.RegistrationInfo.Description = "Performs the SnapRAID Sync command after a small delay after logon";
            td.RegistrationInfo.Documentation = "http://elucidate.codeplex.com/documentation";
            td.Settings.DisallowStartIfOnBatteries = true;
            td.Settings.Enabled = true;
            td.Settings.ExecutionTimeLimit = TimeSpan.FromDays(1);
            td.Settings.Hidden = false;
            td.Settings.Priority = ProcessPriorityClass.Normal;
            td.Settings.RunOnlyIfIdle = false;
            td.Settings.RunOnlyIfNetworkAvailable = false;
            td.Settings.StopIfGoingOnBatteries = true;
            bool newVer = (ts.HighestSupportedVersion >= new Version(1, 2));
            // Create a trigger that fires 1 Minute after the current user logs on
            LogonTrigger lTrigger = (LogonTrigger)td.Triggers.Add(new LogonTrigger());
            if (newVer)
            {
               lTrigger.Delay = TimeSpan.FromMinutes(1);
               lTrigger.UserId = user;
            }
            lTrigger.Repetition.Interval = TimeSpan.FromDays(1);
            // Create an action which opens a log file in notepad
            string appPath;
            string args = FormatSnapRaidCommandArgs("Sync", out appPath);
            td.Actions.Add(new ExecAction("cmd", string.Format("/k \"\"{0}\" {1}\"", appPath, args), null));
            //if (newVer)
            //{
            //   // Create an action which shows a message to the interactive user
            //   td.Actions.Add(new ShowMessageAction("Running Notepad", "Info"));
            //   // Create an action which sends an email
            //   td.Actions.Add(new EmailAction("Testing", "dahall@codeplex.com", "user@test.com", "You've got mail.", "mail.myisp.com"));
            //}
            // Register the task definition (saves it) in the security context of the interactive user
            ts.RootFolder.RegisterTaskDefinition(taskName, td, TaskCreation.CreateOrUpdate, null, null, TaskLogonType.InteractiveToken);
         }
      }

      private void btnNew_Click(object sender, EventArgs e)
      {
         try
         {
            EnableScheduleButtons(false);

            GetTaskTemplate();
            btnGetSchedules_Click(sender, e);
         }
         catch (Exception ex)
         {
            Log.Error("btnNew_Click has thrown: ", ex);
            MessageBox.Show(this, ex.Message, "New Schedule Task");
         }
         finally
         {
            EnableScheduleButtons(true);
         }
      }

      private void btnEdit_Click(object sender, EventArgs e)
      {
         try
         {
            EnableScheduleButtons(false);

            using (TaskService ts = new TaskService())
            {
               Task target = ts.GetTask(taskName);
               if (target == null)
               {
                  GetTaskTemplate();
                  target = ts.GetTask(taskName);
               }
               using (target)
               {
                  //// Edit task and re-register if user clicks Ok
                  using (TaskEditDialog editorForm = new TaskEditDialog(target)
                                                        {
                                                           Editable = true,
                                                           RegisterTaskOnAccept = true
                                                        })
                  {
                     editorForm.AvailableTabs |= AvailableTaskTabs.RunTimes;

                     editorForm.ShowDialog();
                  }
               }
            }
            btnGetSchedules_Click(sender, e);
         }
         catch (Exception ex)
         {
            Log.Error("btnEdit_Click has thrown: ", ex);
            MessageBox.Show(this, ex.Message, "Edit Schedule Task");
         }
         finally
         {
            EnableScheduleButtons(true);
         }
      }

      private void btnDelete_Click(object sender, EventArgs e)
      {
         try
         {
            EnableScheduleButtons(false);

            using (TaskService ts = new TaskService())
            {
               // Remove the task we created
               ts.RootFolder.DeleteTask(taskName);
            }
            btnGetSchedules_Click(sender, e);
         }
         catch (Exception ex)
         {
            Log.Error("btnDelete_Click has thrown: ", ex);
            MessageBox.Show(this, ex.Message, "Delete Schedule Task");
         }
         finally
         {
            EnableScheduleButtons(true);
         }
      }
      #endregion

      private void deleteContentsToolStripMenuItem_Click(object sender, EventArgs e)
      {
         textBox1.Clear();
      }

      #region Recovery options pane
      private void tabControl1_Selected(object sender, TabControlEventArgs e)
      {
         if (e.TabPage == RecoveryOperations)
         {
            WindowState = FormWindowState.Maximized;
            webBrowser1.Url = new Uri("http://snapraid.sourceforge.net/manual.html#9", UriKind.Absolute);
         }
         else if (e.TabPage == coveragePage)
         {
            if (Properties.Settings.Default.ConfigFileIsValid)
               try
               {
                  ConfigFileHelper cfg = new ConfigFileHelper(Properties.Settings.Default.ConfigFileLocation);
                  cfg.Read();
                  List<string> displayLines = cfg.SnapShotSources;
                  displayLines.Add(new FileInfo(cfg.ParityFile).DirectoryName);
                  if (!string.IsNullOrEmpty(cfg.Q_ParityFile))
                  {
                     displayLines.Add(new FileInfo(cfg.Q_ParityFile).DirectoryName);
                  }
                  driveSpace.StartProcessing(displayLines);
               }
               catch { }
         }
      }

      private void tabControl1_Deselected(object sender, TabControlEventArgs e)
      {
         if (e.TabPage == RecoveryOperations)
         {
            if (WindowState == FormWindowState.Maximized)
               WindowState = FormWindowState.Normal;
         }
      }

      private void btnRemoveOutput_Click(object sender, EventArgs e)
      {
         if (DialogResult.Yes == MessageBoxExt.Show(this, "Are you sure you want to perform this task ?",
                        "Remove SnapRAID Output files.", MessageBoxButtons.YesNo, MessageBoxIcon.Question))
            try
            {
               ConfigFileHelper cfg = new ConfigFileHelper(Properties.Settings.Default.ConfigFileLocation);
               string readResult;
               if (!string.IsNullOrEmpty(readResult = cfg.Read()))
                  MessageBoxExt.Show(this, readResult, "Config Read Error:", MessageBoxButtons.OK, MessageBoxIcon.Error);
               else
               {
                  FileInfo fi;
                  if (!string.IsNullOrEmpty(cfg.ParityFile))
                  {
                     fi = new FileInfo(cfg.ParityFile);
                     if (fi.Exists)
                        fi.Delete();
                  }
                  if (!string.IsNullOrEmpty(cfg.Q_ParityFile))
                  {
                     fi = new FileInfo(cfg.Q_ParityFile);
                     if (fi.Exists)
                        fi.Delete();
                  }
                  if (cfg.ContentFiles != null)
                  {
                     foreach (string contentFile in cfg.ContentFiles.Where(contentFile => !string.IsNullOrEmpty(contentFile)))
                     {
                        fi = new FileInfo(contentFile);
                        if (fi.Exists)
                           fi.Delete();
                     }
                  }
               }

            }
            catch (Exception ex)
            {
               Log.Error("btnRemoveOutput_Click has thrown: ", ex);
               MessageBox.Show(this, ex.Message, "Remove SnapRAID Output files.");
            }
      }
      #endregion

      private void timer1_Tick(object sender, EventArgs e)
      {
         try
         {
            if (logs.Count <= 0)
               return;
            // Now lock in case the timer is overlapping !
            lock (this)
            {
               textBox1._Paint = false; // turn off flag to ignore WM_PAINT messages
               if (textBox1.Lines.Length > Properties.Settings.Default.MaxNumberOfRealTimeLines)
               {
                  //textBox1.SelectionStart = 0;
                  //textBox1.SelectionLength = textBox1.GetFirstCharIndexFromLine(Properties.Settings.Default.MaxNumberOfRealTimeLines - 10);
                  //textBox1.SelectedText = string.Empty;
                  // The above makes it beep as well !!
                  textBox1.Clear();
               }
               //read out of the file until the EOF
               while (logs.Count > 0)
               {
                  int textLength = textBox1.TextLength;
                  textBox1.Select(textLength, 0);
                  LogString log = logs.Dequeue();
                  switch (log.LevelUppercase)
                  {
                     case "FATAL":
                        textBox1.SelectionColor = Color.DarkViolet;
                        break;
                     case "ERROR":
                        textBox1.SelectionColor = Color.Red;
                        break;
                     case "WARN":
                        textBox1.SelectionColor = Color.RoyalBlue;
                        break;
                     case "INFO":
                        textBox1.SelectionColor = Color.Black;
                        break;
                     case "DEBUG":
                        textBox1.SelectionColor = Color.DarkGray;
                        break;
                     case "TRACE":
                        textBox1.SelectionColor = Color.DimGray;
                        break;
                     default:
                        // Leave it as is
                        break;
                  }
                  textBox1.AppendText(log.Message + Environment.NewLine);
               }
            }
         }
         catch { }
         textBox1._Paint = true;// restore flag so we can paint the control
      }

      private class LogString
      {
         public string LevelUppercase;
         public string Message;
      };

      static private readonly Queue<LogString> logs = new Queue<LogString>();
      // ReSharper disable UnusedMember.Global
      // This is used by the logging to force all to the output window
      public static void LogMethod(string levelUppercase, string message)
      {
         logs.Enqueue(new LogString
         {
            LevelUppercase = levelUppercase,
            Message = message
         });
      }
      // ReSharper restore UnusedMember.Global
   }
}
